﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using static System.Console;

namespace NetCore_IPAdress
{
    class Program
    {
        static void Main(string[] args)
        {
            //ValidaEnderecoIP();
            //UsandoIPAddress();
            ObtemEnderecoIP();
            ReadLine();
        }

        private static void ObtemEnderecoIP()
        {
            try
            {
                WriteLine("Endereço IP Local : " + GetIPAddress(Dns.GetHostName()));
                WriteLine("Endereço IP do Google:" + GetIPAddress("google.com"));
                WriteLine("Endereço IP da Microsoft:" + GetIPAddress("microsoft.com"));
                WriteLine("Endereço IP do portal Macoratti.net:" + GetIPAddress("macoratti.net"));
                ReadLine();
            }
            catch (Exception ex)
            {
                WriteLine("Erro : " + ex.Message);
            }
        }

        private static void UsandoIPAddress()
        {
            WriteLine("Usando IPAdress\n");
            WriteLine("Convertendo IPv4/IPv6\n");
            string enderecoIP = "65.52.128.33";
            IPAddressConversao(enderecoIP);
            ReadLine();
            WriteLine("Criando endereços\n");
            IPAddressCriaEnderecos(enderecoIP);
        }

        private static void ValidaEnderecoIP()
        {
            IPAddress IP;
            bool flag = false;
            WriteLine("Informe o endereço IP: ");
            string enderecoIP = ReadLine();

            try
            {
                flag = IPAddress.TryParse(enderecoIP, out IP);

                if (flag)
                    WriteLine($"{enderecoIP} é um endereço IP válido");
                else
                    WriteLine($"{enderecoIP} Não é um endereço IP Válido");

            }
            catch (Exception ex)
            {
                throw new Exception("Erro : " + ex.Message);
            }
        }

        public static IPAddress GetIPAddress(string hostName)
        {
            Ping ping = new Ping();
            try
            {
                var replay = ping.Send(hostName);
                if (replay.Status == IPStatus.Success)
                {
                    return replay.Address;
                }
                return null;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public static IEnumerable<string> GetEnderecos()
        {
            var host = Dns.GetHostEntry(Dns.GetHostName());
            return (from ip in host.AddressList where ip.AddressFamily == AddressFamily.InterNetwork
                             select ip.ToString()).ToList();
        }

        public static void IPAddressConversao(string ipAddressString)
        {
            IPAddress address;
            if (!IPAddress.TryParse(ipAddressString, out address))
            {
                WriteLine($"Não foi possível analisar : {ipAddressString}");
                return;
            }
            byte[] bytes = address.GetAddressBytes();
            for (int i = 0; i < bytes.Length; i++)
            {
                WriteLine($"byte {i}: {bytes[i]:X}");
            }
            WriteLine($"familia : {address.AddressFamily}, " + $"mapeia para ipv6: {address.MapToIPv6()}, mapeia para ipv4: {address.MapToIPv4()}");
            //
        }

        public static void IPAddressCriaEnderecos(string ipAddressString)
        {
            WriteLine($"Criando endereços IP a partir de :  {ipAddressString}\n");
            WriteLine($"IPv4 endereço loopback: {IPAddress.Loopback}");
            WriteLine($"IPv6 endereço loopback: {IPAddress.IPv6Loopback}");
            WriteLine($"IPv4 broadcast address: {IPAddress.Broadcast}");
            WriteLine($"IPv4 qualquer endereço: {IPAddress.Any}");
            WriteLine($"IPv6 qualquer endereço: {IPAddress.IPv6Any}");
        }
    }
}
